@@ -63,7 +63,8 @@
 
 
 
-#undef DUMP_IMAGE_DETAILS
+#undef DUMP_IMAGE_DETAILS 
+#undef IO_GIFDEBUG
 
 #define MAXCOLORMAPSIZE  256
 #define MAX_LZW_BITS     12
@@ -83,7 +84,7 @@ enum {
 	GIF_GET_COLORMAP,
 	GIF_GET_NEXT_STEP,
 	GIF_GET_FRAME_INFO,
-	GIF_GET_EXTENTION,
+	GIF_GET_EXTENSION,
 	GIF_GET_COLORMAP2,
 	GIF_PREPARE_LZW,
 	GIF_LZW_FILL_BUFFER,
@@ -149,6 +150,7 @@ struct _GifContext
 	/* extension context */
 	guchar extension_label;
 	guchar extension_flag;
+        gboolean in_loop_extension;
 
 	/* get block context */
 	guchar block_count;
@@ -249,7 +251,7 @@ gif_read (GifContext *context, guchar *buffer, size_t len)
 		}
 		context->amount_needed = len - (context->size - context->ptr);
 	}
-	return 0;
+	return FALSE;
 }
 
 /* Changes the stage to be GIF_GET_COLORMAP */
@@ -353,7 +355,7 @@ get_data_block (GifContext *context,
 static void
 gif_set_get_extension (GifContext *context)
 {
-	context->state = GIF_GET_EXTENTION;
+	context->state = GIF_GET_EXTENSION;
 	context->extension_flag = TRUE;
 	context->extension_label = 0;
 	context->block_count = 0;
@@ -376,8 +378,8 @@ gif_get_extension (GifContext *context)
 		}
 
 		switch (context->extension_label) {
-		case 0xf9:			/* Graphic Control Extension */
-			retval = get_data_block (context, (unsigned char *) context->block_buf, NULL);
+                case 0xf9:			/* Graphic Control Extension */
+                        retval = get_data_block (context, (unsigned char *) context->block_buf, NULL);
 			if (retval != 0)
 				return retval;
 
@@ -399,6 +401,36 @@ gif_get_extension (GifContext *context)
 			/* Now we've successfully loaded this one, we continue on our way */
 			context->block_count = 0;
 			context->extension_flag = FALSE;
+			break;
+                case 0xff: /* application extension */
+                        if (!context->in_loop_extension) { 
+                                retval = get_data_block (context, (unsigned char *) context->block_buf, NULL);
+                                if (retval != 0)
+                                        return retval;
+                                if (!strncmp (context->block_buf, "NETSCAPE2.0", 11) ||
+                                    !strncmp (context->block_buf, "ANIMEXTS1.0", 11)) {
+                                        context->in_loop_extension = TRUE;
+                                }
+                                context->block_count = 0;
+                        }
+                        if (context->in_loop_extension) {
+                                do {
+                                        retval = get_data_block (context, (unsigned char *) context->block_buf, &empty_block);
+                                        if (retval != 0)
+                                                return retval;
+                                        if (context->block_buf[0] == 0x01) {
+                                                context->animation->loop = context->block_buf[1] + (context->block_buf[2] << 8);
+                                                if (context->animation->loop != 0) 
+                                                        context->animation->loop++;
+                                        }
+                                        context->block_count = 0;
+                                }
+                                while (!empty_block);
+                                context->in_loop_extension = FALSE;
+                                context->extension_flag = FALSE;
+                                return 0;
+                        }
+			break;                          
 		default:
 			/* Unhandled extension */
 			break;
@@ -534,6 +566,17 @@ gif_lzw_clear_code (GifContext *context)
 	return 0;
 }
 
+#define CHECK_LZW_SP() G_STMT_START {                                           \
+        if ((guchar *)context->lzw_sp >=                                        \
+            (guchar *)context->lzw_stack + sizeof (context->lzw_stack)) {       \
+                 g_set_error (context->error,                                   \
+                             GDK_PIXBUF_ERROR,                                  \
+                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,                    \
+                             _("Stack overflow"));                              \
+                return -2;                                                      \
+        }                                                                       \
+} G_STMT_END
+
 static int
 lzw_read_byte (GifContext *context)
 {
@@ -608,19 +651,20 @@ lzw_read_byte (GifContext *context)
 		incode = code;
 
 		if (code >= context->lzw_max_code) {
+                        CHECK_LZW_SP ();
 			*(context->lzw_sp)++ = context->lzw_firstcode;
 			code = context->lzw_oldcode;
 		}
 
 		while (code >= context->lzw_clear_code) {
-                        if ((code >= (1 << MAX_LZW_BITS)) 
-                            || (context->lzw_sp >= context->lzw_stack + ((1 << (MAX_LZW_BITS)) * 2 + 1))) {
+                        if (code >= (1 << MAX_LZW_BITS)) {
                                 g_set_error (context->error,
                                              GDK_PIXBUF_ERROR,
                                              GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
                                              _("Bad code encountered"));
 				return -2;
                         }
+                        CHECK_LZW_SP ();
 			*(context->lzw_sp)++ = context->lzw_table[1][code];
 
 			if (code == context->lzw_table[0][code]) {
@@ -633,6 +677,7 @@ lzw_read_byte (GifContext *context)
 			code = context->lzw_table[0][code];
 		}
 
+                CHECK_LZW_SP ();
 		*(context->lzw_sp)++ = context->lzw_firstcode = context->lzw_table[1][code];
 
 		if ((code = context->lzw_max_code) < (1 << MAX_LZW_BITS)) {
@@ -748,12 +793,31 @@ gif_get_lzw (GifContext *context)
                 context->frame->composited = NULL;
                 context->frame->revert = NULL;
                 
-                context->frame->pixbuf =
-                        gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-                                        TRUE,
-                                        8,
-                                        context->frame_len,
-                                        context->frame_height);
+                if (context->frame_len == 0 || context->frame_height == 0) {
+                        /* An empty frame, we just output a single transparent
+                         * pixel at (0, 0).
+                         */
+                        context->x_offset = 0;
+                        context->y_offset = 0;
+                        context->frame_len = 1;
+                        context->frame_height = 1;
+                        context->frame->pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 1, 1);
+                        if (context->frame->pixbuf) {
+                                guchar *pixels;
+
+                                pixels = gdk_pixbuf_get_pixels (context->frame->pixbuf);
+                                pixels[0] = 0;
+                                pixels[1] = 0;
+                                pixels[2] = 0;
+                                pixels[3] = 0;
+                        }
+                } else
+                        context->frame->pixbuf =
+                                gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+                                                TRUE,
+                                                8,
+                                                context->frame_len,
+                                                context->frame_height);
                 if (!context->frame->pixbuf) {
                         g_free (context->frame);
                         g_set_error (context->error,
@@ -943,7 +1007,8 @@ gif_get_lzw (GifContext *context)
 		if (lower_bound <= upper_bound && first_pass == context->draw_pass) {
 			(* context->update_func)
 				(context->frame->pixbuf,
-				 0, lower_bound,
+                                 context->frame->x_offset,
+                                 context->frame->y_offset + lower_bound,
 				 gdk_pixbuf_get_width (context->frame->pixbuf),
 				 upper_bound - lower_bound,
 				 context->user_data);
@@ -967,9 +1032,9 @@ gif_get_lzw (GifContext *context)
 				(* context->update_func)
 					(context->frame->pixbuf,
 					 context->frame->x_offset,
-                                         lower_bound + context->frame->y_offset,
+                                         context->frame->y_offset + lower_bound,
 					 gdk_pixbuf_get_width (context->frame->pixbuf),
-					 gdk_pixbuf_get_height (context->frame->pixbuf),
+					 gdk_pixbuf_get_height (context->frame->pixbuf) - lower_bound,
 					 context->user_data);
 			}
 		}
@@ -1135,17 +1200,6 @@ gif_get_frame_info (GifContext *context)
 	context->x_offset = LM_to_uint (buf[0], buf[1]);
 	context->y_offset = LM_to_uint (buf[2], buf[3]);
 
-        if ((context->frame_height == 0) || (context->frame_len == 0)) {
-		context->state = GIF_DONE;
-
-                g_set_error (context->error,
-                             GDK_PIXBUF_ERROR,
-                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                             _("GIF image contained a frame with height or width 0."));
-                
-		return -2;
-        }
-            
 	if (((context->frame_height + context->y_offset) > context->height) ||
             ((context->frame_len + context->x_offset) > context->width)) {
 		/* All frames must fit in the image bounds */
@@ -1162,26 +1216,19 @@ gif_get_frame_info (GifContext *context)
 	if (context->animation->frames == NULL &&
             context->gif89.disposal == 3) {
                 /* First frame can't have "revert to previous" as its
-                 * dispose mode.
+                 * dispose mode. Silently use "retain" instead. 
                  */
-                
-		context->state = GIF_DONE;
-
-                g_set_error (context->error,
-                             GDK_PIXBUF_ERROR,
-                             GDK_PIXBUF_ERROR_CORRUPT_IMAGE,
-                             _("First frame of GIF image had 'revert to previous' as its disposal mode."));
-                
-		return -2;
+                context->gif89.disposal = 0;
 	}
 
+	context->frame_interlace = BitSet (buf[8], INTERLACE);
+
 #ifdef DUMP_IMAGE_DETAILS
-        g_print (">width: %d height: %d xoffset: %d yoffset: %d disposal: %d delay: %d transparent: %d\n",
+        g_print (">width: %d height: %d xoffset: %d yoffset: %d disposal: %d delay: %d transparent: %d interlace: %d\n",
                  context->frame_len, context->frame_height, context->x_offset, context->y_offset,
-                 context->gif89.disposal, context->gif89.delay_time, context->gif89.transparent);
+                 context->gif89.disposal, context->gif89.delay_time, context->gif89.transparent, context->frame_interlace);
 #endif
         
-	context->frame_interlace = BitSet (buf[8], INTERLACE);
 	if (BitSet (buf[8], LOCALCOLORMAP)) {
 
 #ifdef DUMP_IMAGE_DETAILS
@@ -1280,7 +1327,7 @@ gif_main_loop (GifContext *context)
 			retval = gif_get_frame_info (context);
 			break;
 
-		case GIF_GET_EXTENTION:
+		case GIF_GET_EXTENSION:
                         LOG("get_extension\n");
 			retval = gif_get_extension (context);
 			if (retval == 0)
@@ -1349,6 +1396,8 @@ new_context (void)
 	context->gif89.delay_time = -1;
 	context->gif89.input_flag = -1;
 	context->gif89.disposal = -1;
+        context->animation->loop = 1;
+        context->in_loop_extension = FALSE;
 
 	return context;
 }
