@@ -337,13 +337,6 @@ GdkImage *_gdk_win32_copy_to_image      (GdkDrawable *drawable,
 					 gint         width,
 					 gint         height);
 
-GdkImage *_gdk_win32_setup_pixmap_image (GdkPixmap   *pixmap,
-					 GdkDrawable *drawable,
-					 gint         width,
-					 gint         height,
-					 gint         depth,
-					 guchar      *bits);
-
 void      _gdk_win32_blit               (gboolean              use_fg_bg,
 					 GdkDrawableImplWin32 *drawable,
 					 GdkGC       	       *gc,
@@ -364,6 +357,12 @@ HRGN	  _gdk_win32_gdkregion_to_hrgn  (GdkRegion   *region,
 					 gint         x_origin,
 					 gint         y_origin);
 
+void	_gdk_win32_adjust_client_rect   (GdkWindow *window,
+					 RECT      *RECT);
+
+void	_gdk_win32_get_adjusted_client_rect (GdkWindow *window,
+					     RECT      *RECT);
+
 void    _gdk_selection_property_store (GdkWindow *owner,
                                        GdkAtom    type,
                                        gint       format,
@@ -460,6 +459,16 @@ extern GdkWindow        *_gdk_parent_root;
 extern GdkDisplay       *_gdk_display;
 extern GdkScreen        *_gdk_screen;
 
+extern gint		 _gdk_num_monitors;
+extern GdkRectangle     *_gdk_monitors;
+
+/* Offsets to add to Windows coordinates (which are relative to the
+ * primary monitor's origin, and thus might be negative for monitors
+ * to the left and/or above the primary monitor) to get GDK
+ * coordinates, which should be non-negative on the whole screen.
+ */
+extern gint		 _gdk_offset_x, _gdk_offset_y;
+
 extern HDC		 _gdk_display_hdc;
 extern HINSTANCE	 _gdk_dll_hinstance;
 extern HINSTANCE	 _gdk_app_hmodule;
@@ -468,9 +477,11 @@ extern HINSTANCE	 _gdk_app_hmodule;
  * from a single thread anyway.
  */
 extern HKL		 _gdk_input_locale;
+extern gboolean		 _gdk_input_locale_is_ime;
 extern UINT		 _gdk_input_codepage;
 
 extern guint		 _gdk_keymap_serial;
+extern gboolean		 _gdk_keyboard_has_altgr;
 
 /* Registered clipboard formats */
 extern WORD		 _cf_rtf;
