@@ -267,7 +267,7 @@ _gdk_input_common_find_events(GdkWindow *window,
   XEventClass class;
   
   i = 0;
-  if (mask & GDK_BUTTON_PRESS_MASK || mask & GDK_BUTTON_RELEASE_MASK)
+  if (mask & GDK_BUTTON_PRESS_MASK)
     {
       DeviceButtonPress (gdkdev->xdevice, gdkdev->buttonpress_type,
 			     class);
@@ -277,6 +277,13 @@ _gdk_input_common_find_events(GdkWindow *window,
       if (class != 0)
 	  classes[i++] = class;
     }
+  if (mask & GDK_BUTTON_RELEASE_MASK)
+    {
+      DeviceButtonRelease (gdkdev->xdevice, gdkdev->buttonrelease_type,
+			   class);
+      if (class != 0)
+	  classes[i++] = class;
+    }
   if (mask & GDK_POINTER_MOTION_MASK)
     {
       DeviceMotionNotify  (gdkdev->xdevice, gdkdev->motionnotify_type, class);
@@ -378,18 +385,17 @@ _gdk_input_common_init (GdkDisplay *display,
   XDeviceInfo   *devices;
   int num_devices;
   int num_extensions, loop;
+  int ignore, event_base;
   GdkDisplayX11 *display_x11 = GDK_DISPLAY_X11 (display);
 
   /* Init XInput extension */
-  
-  extensions = XListExtensions(display_x11->xdisplay, &num_extensions);
-  for (loop = 0; loop < num_extensions &&
-	 (strcmp(extensions[loop], "XInputExtension") != 0); loop++);
-  XFreeExtensionList(extensions);
+
   display_x11->input_devices = NULL;
-  if (loop < num_extensions)
+  if (XQueryExtension (display_x11->xdisplay, "XInputExtension",
+		       &ignore, &event_base, &ignore))
     {
-      /* XInput extension found */
+      _gdk_x11_register_event_type (display,
+				    event_base, 9 /* Number of events */);
 
       devices = XListInputDevices(display_x11->xdisplay, &num_devices);
   
@@ -520,7 +526,7 @@ gdk_input_translate_state(guint state, guint device_state)
 }
 
 
-gint 
+gboolean
 _gdk_input_common_other_event (GdkEvent         *event,
 			       XEvent           *xevent,
 			       GdkInputWindow   *input_window,
@@ -667,7 +673,7 @@ _gdk_input_common_other_event (GdkEvent         *event,
       return TRUE;
   }
 
-  return -1;			/* wasn't one of our event types */
+  return FALSE;			/* wasn't one of our event types */
 }
 
 gboolean
