@@ -26,62 +26,64 @@
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
+
+#include <string.h>
+
 #include "gtkaccellabel.h"
 #include "gtkaccelmap.h"
 #include "gtkmain.h"
-#include "gtksignal.h"
 #include "gtkintl.h"
 
-#include <string.h>
-
 enum {
   PROP_0,
   PROP_ACCEL_CLOSURE,
   PROP_ACCEL_WIDGET
 };
 
-static void     gtk_accel_label_class_init   (GtkAccelLabelClass *klass);
-static void     gtk_accel_label_init         (GtkAccelLabel      *accel_label);
-static void     gtk_accel_label_set_property (GObject            *object,
-					      guint               prop_id,
-					      const GValue       *value,
-					      GParamSpec         *pspec);
-static void     gtk_accel_label_get_property (GObject            *object,
-					      guint               prop_id,
-					      GValue             *value,
-					      GParamSpec         *pspec);
-static void     gtk_accel_label_destroy      (GtkObject          *object);
-static void     gtk_accel_label_finalize     (GObject            *object);
-static void     gtk_accel_label_size_request (GtkWidget          *widget,
-					      GtkRequisition     *requisition);
-static gboolean gtk_accel_label_expose_event (GtkWidget          *widget,
-					      GdkEventExpose     *event);
-static gboolean gtk_accel_label_refetch_idle (GtkAccelLabel      *accel_label);
+static void         gtk_accel_label_class_init   (GtkAccelLabelClass *klass);
+static void         gtk_accel_label_init         (GtkAccelLabel      *accel_label);
+static void         gtk_accel_label_set_property (GObject            *object,
+						  guint               prop_id,
+						  const GValue       *value,
+						  GParamSpec         *pspec);
+static void         gtk_accel_label_get_property (GObject            *object,
+						  guint               prop_id,
+						  GValue             *value,
+						  GParamSpec         *pspec);
+static void         gtk_accel_label_destroy      (GtkObject          *object);
+static void         gtk_accel_label_finalize     (GObject            *object);
+static void         gtk_accel_label_size_request (GtkWidget          *widget,
+						  GtkRequisition     *requisition);
+static gboolean     gtk_accel_label_expose_event (GtkWidget          *widget,
+						  GdkEventExpose     *event);
+static const gchar *gtk_accel_label_get_string   (GtkAccelLabel      *accel_label);
+
 
-static GtkAccelLabelClass *accel_label_class = NULL;
 static GtkLabelClass *parent_class = NULL;
 
-
-GtkType
+GType
 gtk_accel_label_get_type (void)
 {
-  static GtkType accel_label_type = 0;
+  static GType accel_label_type = 0;
   
   if (!accel_label_type)
     {
-      static const GtkTypeInfo accel_label_info =
+      static const GTypeInfo accel_label_info =
       {
-	"GtkAccelLabel",
-	sizeof (GtkAccelLabel),
 	sizeof (GtkAccelLabelClass),
-	(GtkClassInitFunc) gtk_accel_label_class_init,
-	(GtkObjectInitFunc) gtk_accel_label_init,
-        /* reserved_1 */ NULL,
-	/* reserved_2 */ NULL,
-	(GtkClassInitFunc) NULL,
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_accel_label_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkAccelLabel),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_accel_label_init,
       };
       
-      accel_label_type = gtk_type_unique (GTK_TYPE_LABEL, &accel_label_info);
+      accel_label_type =
+	g_type_register_static (GTK_TYPE_LABEL, "GtkAccelLabel",
+				&accel_label_info, 0);
     }
   
   return accel_label_type;
@@ -94,7 +96,6 @@ gtk_accel_label_class_init (GtkAccelLabelClass *class)
   GtkObjectClass *object_class = GTK_OBJECT_CLASS (class);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (class);
   
-  accel_label_class = class;
   parent_class = g_type_class_peek_parent (class);
   
   gobject_class->finalize = gtk_accel_label_finalize;
@@ -108,21 +109,36 @@ gtk_accel_label_class_init (GtkAccelLabelClass *class)
 
   class->signal_quote1 = g_strdup ("<:");
   class->signal_quote2 = g_strdup (":>");
-  class->mod_name_shift = g_strdup ("Shift");
-  class->mod_name_control = g_strdup ("Ctrl");
-  class->mod_name_alt = g_strdup ("Alt");
+  /* This is the text that should appear next to menu accelerators
+   * that use the shift key. If the text on this key isn't typically
+   * translated on keyboards used for your language, don't translate
+   * this.
+   */
+  class->mod_name_shift = g_strdup (_("Shift"));
+  /* This is the text that should appear next to menu accelerators
+   * that use the control key. If the text on this key isn't typically
+   * translated on keyboards used for your language, don't translate
+   * this.
+   */
+  class->mod_name_control = g_strdup (_("Ctrl"));
+  /* This is the text that should appear next to menu accelerators
+   * that use the alt key. If the text on this key isn't typically
+   * translated on keyboards used for your language, don't translate
+   * this.
+   */
+  class->mod_name_alt = g_strdup (_("Alt"));
   class->mod_separator = g_strdup ("+");
   class->accel_seperator = g_strdup (" / ");
   class->latin1_to_char = TRUE;
   
-  g_object_class_install_property (G_OBJECT_CLASS (object_class),
+  g_object_class_install_property (gobject_class,
                                    PROP_ACCEL_CLOSURE,
                                    g_param_spec_boxed ("accel_closure",
 						       _("Accelerator Closure"),
 						       _("The closure to be monitored for accelerator changes"),
 						       G_TYPE_CLOSURE,
 						       G_PARAM_READABLE | G_PARAM_WRITABLE));
-  g_object_class_install_property (G_OBJECT_CLASS (object_class),
+  g_object_class_install_property (gobject_class,
                                    PROP_ACCEL_WIDGET,
                                    g_param_spec_object ("accel_widget",
                                                         _("Accelerator Widget"),
@@ -182,14 +198,11 @@ gtk_accel_label_get_property (GObject    *object,
 static void
 gtk_accel_label_init (GtkAccelLabel *accel_label)
 {
-  accel_label->queue_id = 0;
   accel_label->accel_padding = 3;
   accel_label->accel_widget = NULL;
   accel_label->accel_closure = NULL;
   accel_label->accel_group = NULL;
   accel_label->accel_string = NULL;
-  
-  gtk_accel_label_refetch (accel_label);
 }
 
 GtkWidget*
@@ -199,7 +212,7 @@ gtk_accel_label_new (const gchar *string)
   
   g_return_val_if_fail (string != NULL, NULL);
   
-  accel_label = gtk_type_new (GTK_TYPE_ACCEL_LABEL);
+  accel_label = g_object_new (GTK_TYPE_ACCEL_LABEL, NULL);
   
   gtk_label_set_text (GTK_LABEL (accel_label), string);
   
@@ -222,11 +235,6 @@ gtk_accel_label_finalize (GObject *object)
 {
   GtkAccelLabel *accel_label = GTK_ACCEL_LABEL (object);
 
-  if (accel_label->queue_id)
-    {
-      gtk_idle_remove (accel_label->queue_id);
-      accel_label->queue_id = 0;
-    }
   g_free (accel_label->accel_string);
   
   G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -270,11 +278,24 @@ gtk_accel_label_size_request (GtkWidget	     *widget,
   if (GTK_WIDGET_CLASS (parent_class)->size_request)
     GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
 
-  layout = gtk_widget_create_pango_layout (widget, accel_label->accel_string);
+  layout = gtk_widget_create_pango_layout (widget, gtk_accel_label_get_string (accel_label));
   pango_layout_get_pixel_size (layout, &width, NULL);
   accel_label->accel_string_width = width;
   
-  g_object_unref (G_OBJECT (layout));
+  g_object_unref (layout);
+}
+
+static gint
+get_first_baseline (PangoLayout *layout)
+{
+  PangoLayoutIter *iter;
+  gint result;
+
+  iter = pango_layout_get_iter (layout);
+  result = pango_layout_iter_get_baseline (iter);
+  pango_layout_iter_free (iter);
+
+  return PANGO_PIXELS (result);
 }
 
 static gboolean 
@@ -283,8 +304,10 @@ gtk_accel_label_expose_event (GtkWidget      *widget,
 {
   GtkAccelLabel *accel_label = GTK_ACCEL_LABEL (widget);
   GtkMisc *misc = GTK_MISC (accel_label);
-  PangoLayout *layout;
-	  
+  GtkTextDirection direction;
+
+  direction = gtk_widget_get_direction (widget);
+
   if (GTK_WIDGET_DRAWABLE (accel_label))
     {
       guint ac_width;
@@ -293,22 +316,32 @@ gtk_accel_label_expose_event (GtkWidget      *widget,
       
       if (widget->allocation.width >= widget->requisition.width + ac_width)
 	{
-	  guint x;
-	  guint y;
+	  PangoLayout *label_layout;
+	  PangoLayout *accel_layout;
+
+	  gint x;
+	  gint y;
 	  
+	  if (direction == GTK_TEXT_DIR_RTL)
+	    widget->allocation.x += ac_width;
 	  widget->allocation.width -= ac_width;
 	  if (GTK_WIDGET_CLASS (parent_class)->expose_event)
 	    GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+	  if (direction == GTK_TEXT_DIR_RTL)
+	    widget->allocation.x -= ac_width;
 	  widget->allocation.width += ac_width;
 	  
-	  x = widget->allocation.x + widget->allocation.width - misc->xpad - ac_width;
-	  
-	  y = (widget->allocation.y * (1.0 - misc->yalign) +
-	       (widget->allocation.y + widget->allocation.height -
-		(widget->requisition.height - misc->ypad * 2)) *
-	       misc->yalign) + 1.5;
-	  
-	  layout = gtk_widget_create_pango_layout (widget, accel_label->accel_string);
+	  if (direction == GTK_TEXT_DIR_RTL)
+	    x = widget->allocation.x + misc->xpad;
+	  else
+	    x = widget->allocation.x + widget->allocation.width - misc->xpad - ac_width;
+
+	  label_layout = gtk_label_get_layout (GTK_LABEL (accel_label));
+	  gtk_label_get_layout_offsets (GTK_LABEL (accel_label), NULL, &y);
+
+	  accel_layout = gtk_widget_create_pango_layout (widget, gtk_accel_label_get_string (accel_label));
+
+	  y += get_first_baseline (label_layout) - get_first_baseline (accel_layout);
 
           gtk_paint_layout (widget->style,
                             widget->window,
@@ -318,9 +351,9 @@ gtk_accel_label_expose_event (GtkWidget      *widget,
                             widget,
                             "accellabel",
                             x, y,
-                            layout);                            
+                            accel_layout);                            
 
-          g_object_unref (G_OBJECT (layout));
+          g_object_unref (accel_layout);
 	}
       else
 	{
@@ -373,7 +406,7 @@ gtk_accel_label_set_accel_widget (GtkAccelLabel *accel_label,
 	{
 	  gtk_accel_label_set_accel_closure (accel_label, NULL);
 	  g_signal_handlers_disconnect_by_func (accel_label->accel_widget,
-						G_CALLBACK (refetch_widget_accel_closure),
+						refetch_widget_accel_closure,
 						accel_label);
 	  g_object_unref (accel_label->accel_widget);
 	}
@@ -391,14 +424,15 @@ gtk_accel_label_set_accel_widget (GtkAccelLabel *accel_label,
 }
 
 static void
-gtk_accel_label_queue_refetch (GtkAccelLabel *accel_label)
+gtk_accel_label_reset (GtkAccelLabel *accel_label)
 {
-  g_return_if_fail (GTK_IS_ACCEL_LABEL (accel_label));
-
-  if (accel_label->queue_id == 0)
-    accel_label->queue_id = gtk_idle_add_priority (G_PRIORITY_HIGH_IDLE,
-						   (GtkFunction) gtk_accel_label_refetch_idle,
-						   accel_label);
+  if (accel_label->accel_string)
+    {
+      g_free (accel_label->accel_string);
+      accel_label->accel_string = NULL;
+    }
+  
+  gtk_widget_queue_resize (GTK_WIDGET (accel_label));
 }
 
 static void
@@ -409,7 +443,7 @@ check_accel_changed (GtkAccelGroup  *accel_group,
 		     GtkAccelLabel  *accel_label)
 {
   if (accel_closure == accel_label->accel_closure)
-    gtk_accel_label_queue_refetch (accel_label);
+    gtk_accel_label_reset (accel_label);
 }
 
 /**
@@ -433,7 +467,7 @@ gtk_accel_label_set_accel_closure (GtkAccelLabel *accel_label,
       if (accel_label->accel_closure)
 	{
 	  g_signal_handlers_disconnect_by_func (accel_label->accel_group,
-						G_CALLBACK (check_accel_changed),
+						check_accel_changed,
 						accel_label);
 	  accel_label->accel_group = NULL;
 	  g_closure_unref (accel_label->accel_closure);
@@ -447,23 +481,11 @@ gtk_accel_label_set_accel_closure (GtkAccelLabel *accel_label,
 				   G_CALLBACK (check_accel_changed),
 				   accel_label, 0);
 	}
-      gtk_accel_label_queue_refetch (accel_label);
+      gtk_accel_label_reset (accel_label);
       g_object_notify (G_OBJECT (accel_label), "accel_closure");
     }
 }
 
-static gboolean
-gtk_accel_label_refetch_idle (GtkAccelLabel *accel_label)
-{
-  gboolean retval;
-
-  GDK_THREADS_ENTER ();
-  retval = gtk_accel_label_refetch (accel_label);
-  GDK_THREADS_LEAVE ();
-
-  return retval;
-}
-
 static gboolean
 find_accel (GtkAccelKey *key,
 	    GClosure    *closure,
@@ -472,6 +494,15 @@ find_accel (GtkAccelKey *key,
   return data == (gpointer) closure;
 }
 
+static const gchar *
+gtk_accel_label_get_string (GtkAccelLabel *accel_label)
+{
+  if (!accel_label->accel_string)
+    gtk_accel_label_refetch (accel_label);
+  
+  return accel_label->accel_string;
+}
+
 gboolean
 gtk_accel_label_refetch (GtkAccelLabel *accel_label)
 {
@@ -481,8 +512,11 @@ gtk_accel_label_refetch (GtkAccelLabel *accel_label)
 
   class = GTK_ACCEL_LABEL_GET_CLASS (accel_label);
 
-  g_free (accel_label->accel_string);
-  accel_label->accel_string = NULL;
+  if (accel_label->accel_string)
+    {
+      g_free (accel_label->accel_string);
+      accel_label->accel_string = NULL;
+    }
 
   if (accel_label->accel_closure)
     {
@@ -492,6 +526,7 @@ gtk_accel_label_refetch (GtkAccelLabel *accel_label)
 	{
 	  GString *gstring;
 	  gboolean seen_mod = FALSE;
+	  gunichar ch;
 	  
 	  gstring = g_string_new (accel_label->accel_string);
 	  g_string_append (gstring, gstring->len ? class->accel_seperator : "   ");
@@ -517,12 +552,12 @@ gtk_accel_label_refetch (GtkAccelLabel *accel_label)
 	    }
 	  if (seen_mod)
 	    g_string_append (gstring, class->mod_separator);
-	  if (key->accel_key < 0x80 ||
-	      (key->accel_key > 0x80 &&
-	       key->accel_key <= 0xff &&
-	       class->latin1_to_char))
+
+	  ch = gdk_keyval_to_unicode (key->accel_key);
+	  if (ch && (g_unichar_isgraph (ch) || ch == ' ') &&
+	      (ch < 0x80 || class->latin1_to_char))
 	    {
-	      switch (key->accel_key)
+	      switch (ch)
 		{
 		case ' ':
 		  g_string_append (gstring, "Space");
@@ -531,7 +566,7 @@ gtk_accel_label_refetch (GtkAccelLabel *accel_label)
 		  g_string_append (gstring, "Backslash");
 		  break;
 		default:
-		  g_string_append_unichar (gstring, g_unichar_toupper (key->accel_key));
+		  g_string_append_unichar (gstring, g_unichar_toupper (ch));
 		  break;
 		}
 	    }
@@ -541,7 +576,7 @@ gtk_accel_label_refetch (GtkAccelLabel *accel_label)
 	      
 	      tmp = gtk_accelerator_name (key->accel_key, 0);
 	      if (tmp[0] != 0 && tmp[1] == 0)
-		tmp[0] = g_unichar_totitle (tmp[0]);
+		tmp[0] = g_ascii_toupper (tmp[0]);
 	      g_string_append (gstring, tmp);
 	      g_free (tmp);
 	    }
@@ -556,12 +591,6 @@ gtk_accel_label_refetch (GtkAccelLabel *accel_label)
   if (!accel_label->accel_string)
     accel_label->accel_string = g_strdup ("");
 
-  if (accel_label->queue_id)
-    {
-      gtk_idle_remove (accel_label->queue_id);
-      accel_label->queue_id = 0;
-    }
-
   gtk_widget_queue_resize (GTK_WIDGET (accel_label));
 
   return FALSE;
