@@ -1562,6 +1562,7 @@ gtk_icon_set_unref (GtkIconSet *icon_set)
 
           tmp_list = g_slist_next (tmp_list);
         }
+      g_slist_free (icon_set->sources);
 
       clear_cache (icon_set, TRUE);
 
@@ -2435,7 +2436,10 @@ static void
 ensure_cache_up_to_date (GtkIconSet *icon_set)
 {
   if (icon_set->cache_serial != cache_serial)
-    clear_cache (icon_set, TRUE);
+    {
+      clear_cache (icon_set, TRUE);
+      icon_set->cache_serial = cache_serial;
+    }
 }
 
 static void
