@@ -17,13 +17,25 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#include "config.h"
+
 #include <string.h>
 #include <locale.h>
 
-#include "gtksignal.h"
 #include "gtkimmulticontext.h"
 #include "gtkimmodule.h"
 #include "gtkradiomenuitem.h"
+#include "gtkintl.h"
+
+struct _GtkIMMulticontextPrivate
+{
+  GdkWindow *client_window;
+  GdkRectangle cursor_location;
+
+  guint use_preedit : 1;
+  guint have_cursor_location : 1;
+  guint focus_in : 1;
+};
 
 static void     gtk_im_multicontext_class_init         (GtkIMMulticontextClass  *class);
 static void     gtk_im_multicontext_init               (GtkIMMulticontext       *im_multicontext);
@@ -75,10 +87,10 @@ static GtkIMContextClass *parent_class;
 
 static const gchar *global_context_id = NULL;
 
-GtkType
+GType
 gtk_im_multicontext_get_type (void)
 {
-  static GtkType im_multicontext_type = 0;
+  static GType im_multicontext_type = 0;
  
   if (!im_multicontext_type)
     {
@@ -95,9 +107,9 @@ gtk_im_multicontext_get_type (void)
         (GInstanceInitFunc) gtk_im_multicontext_init,
       };
       
-      im_multicontext_type = g_type_register_static (GTK_TYPE_IM_CONTEXT,
-						     "GtkIMMulticontext",
-						     &im_multicontext_info, 0);
+      im_multicontext_type =
+	g_type_register_static (GTK_TYPE_IM_CONTEXT, "GtkIMMulticontext",
+				&im_multicontext_info, 0);
     }
 
   return im_multicontext_type;
@@ -129,6 +141,11 @@ static void
 gtk_im_multicontext_init (GtkIMMulticontext *multicontext)
 {
   multicontext->slave = NULL;
+  
+  multicontext->priv = g_new0 (GtkIMMulticontextPrivate, 1);
+  multicontext->priv->use_preedit = TRUE;
+  multicontext->priv->have_cursor_location = FALSE;
+  multicontext->priv->focus_in = FALSE;
 }
 
 /**
@@ -141,13 +158,16 @@ gtk_im_multicontext_init (GtkIMMulticontext *multicontext)
 GtkIMContext *
 gtk_im_multicontext_new (void)
 {
-  return GTK_IM_CONTEXT (g_object_new (GTK_TYPE_IM_MULTICONTEXT, NULL));
+  return g_object_new (GTK_TYPE_IM_MULTICONTEXT, NULL);
 }
 
 static void
 gtk_im_multicontext_finalize (GObject *object)
 {
-  gtk_im_multicontext_set_slave (GTK_IM_MULTICONTEXT (object), NULL, TRUE);
+  GtkIMMulticontext *multicontext = GTK_IM_MULTICONTEXT (object);
+  
+  gtk_im_multicontext_set_slave (multicontext, NULL, TRUE);
+  g_free (multicontext->priv);
 
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -157,6 +177,7 @@ gtk_im_multicontext_set_slave (GtkIMMulticontext *multicontext,
 			       GtkIMContext      *slave,
 			       gboolean           finalizing)
 {
+  GtkIMMulticontextPrivate *priv = multicontext->priv;
   gboolean need_preedit_changed = FALSE;
   
   if (multicontext->slave)
@@ -165,16 +186,16 @@ gtk_im_multicontext_set_slave (GtkIMMulticontext *multicontext,
 	gtk_im_context_reset (multicontext->slave);
       
       g_signal_handlers_disconnect_by_func (multicontext->slave,
-					    (gpointer) gtk_im_multicontext_preedit_start_cb,
+					    gtk_im_multicontext_preedit_start_cb,
 					    multicontext);
       g_signal_handlers_disconnect_by_func (multicontext->slave,
-					    (gpointer) gtk_im_multicontext_preedit_end_cb,
+					    gtk_im_multicontext_preedit_end_cb,
 					    multicontext);
       g_signal_handlers_disconnect_by_func (multicontext->slave,
-					    (gpointer) gtk_im_multicontext_preedit_changed_cb,
+					    gtk_im_multicontext_preedit_changed_cb,
 					    multicontext);
       g_signal_handlers_disconnect_by_func (multicontext->slave,
-					    (gpointer) gtk_im_multicontext_commit_cb,
+					    gtk_im_multicontext_commit_cb,
 					    multicontext);
 
       g_object_unref (multicontext->slave);
@@ -209,8 +230,14 @@ gtk_im_multicontext_set_slave (GtkIMMulticontext *multicontext,
 			G_CALLBACK (gtk_im_multicontext_delete_surrounding_cb),
 			multicontext);
       
-      if (multicontext->client_window)
-	gtk_im_context_set_client_window (slave, multicontext->client_window);
+      if (!priv->use_preedit)	/* Default is TRUE */
+	gtk_im_context_set_use_preedit (slave, FALSE);
+      if (priv->client_window)
+	gtk_im_context_set_client_window (slave, priv->client_window);
+      if (priv->have_cursor_location)
+	gtk_im_context_set_cursor_location (slave, &priv->cursor_location);
+      if (priv->focus_in)
+	gtk_im_context_focus_in (slave);
     }
 
   if (need_preedit_changed)
@@ -228,11 +255,8 @@ gtk_im_multicontext_get_slave (GtkIMMulticontext *multicontext)
 	{
 	  const char *locale;
 	  
-#ifdef HAVE_LC_MESSAGES
-	  locale = setlocale (LC_MESSAGES, NULL);
-#else
 	  locale = setlocale (LC_CTYPE, NULL);
-#endif
+
 	  global_context_id = _gtk_im_module_get_default_context_id (locale);
 	}
 	
@@ -251,9 +275,10 @@ gtk_im_multicontext_set_client_window (GtkIMContext *context,
 				       GdkWindow    *window)
 {
   GtkIMMulticontext *multicontext = GTK_IM_MULTICONTEXT (context);
+  
   GtkIMContext *slave = gtk_im_multicontext_get_slave (multicontext);
 
-  multicontext->client_window = window;
+  multicontext->priv->client_window = window;
   
   if (slave)
     gtk_im_context_set_client_window (slave, window);
@@ -307,7 +332,9 @@ gtk_im_multicontext_focus_in (GtkIMContext   *context)
     gtk_im_multicontext_set_slave (multicontext, NULL, FALSE);
 
   slave = gtk_im_multicontext_get_slave (multicontext);
-
+  
+  multicontext->priv->focus_in = TRUE;
+  
   if (slave)
     gtk_im_context_focus_in (slave);
 }
@@ -318,6 +345,8 @@ gtk_im_multicontext_focus_out (GtkIMContext   *context)
   GtkIMMulticontext *multicontext = GTK_IM_MULTICONTEXT (context);
   GtkIMContext *slave = gtk_im_multicontext_get_slave (multicontext);
 
+  multicontext->priv->focus_in = FALSE;
+  
   if (slave)
     gtk_im_context_focus_out (slave);
 }
@@ -339,6 +368,9 @@ gtk_im_multicontext_set_cursor_location (GtkIMContext   *context,
   GtkIMMulticontext *multicontext = GTK_IM_MULTICONTEXT (context);
   GtkIMContext *slave = gtk_im_multicontext_get_slave (multicontext);
 
+  multicontext->priv->have_cursor_location = TRUE;
+  multicontext->priv->cursor_location = *area;
+
   if (slave)
     gtk_im_context_set_cursor_location (slave, area);
 }
@@ -350,6 +382,10 @@ gtk_im_multicontext_set_use_preedit (GtkIMContext   *context,
   GtkIMMulticontext *multicontext = GTK_IM_MULTICONTEXT (context);
   GtkIMContext *slave = gtk_im_multicontext_get_slave (multicontext);
 
+  use_preedit = use_preedit != FALSE;
+
+  multicontext->priv->use_preedit = use_preedit;
+
   if (slave)
     gtk_im_context_set_use_preedit (slave, use_preedit);
 }
@@ -448,7 +484,7 @@ activate_cb (GtkWidget         *menuitem,
 {
   if (GTK_CHECK_MENU_ITEM (menuitem)->active)
     {
-      const gchar *id = gtk_object_get_data (GTK_OBJECT (menuitem), "gtk-context-id");
+      const gchar *id = g_object_get_data (G_OBJECT (menuitem), "gtk-context-id");
 
       gtk_im_context_reset (GTK_IM_CONTEXT (context));
       
@@ -471,7 +507,7 @@ gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
 				      GtkMenuShell      *menushell)
 {
   const GtkIMContextInfo **contexts;
-  gint n_contexts, i;
+  guint n_contexts, i;
   GSList *group = NULL;
   
   _gtk_im_module_list (&contexts, &n_contexts);
@@ -479,9 +515,36 @@ gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
   for (i=0; i < n_contexts; i++)
     {
       GtkWidget *menuitem;
-
+      const gchar *translated_name;
+#ifdef ENABLE_NLS
+      if (contexts[i]->domain && contexts[i]->domain_dirname &&
+	  contexts[i]->domain[0] && contexts[i]->domain_dirname[0])
+	{
+	  if (strcmp (contexts[i]->domain, GETTEXT_PACKAGE) == 0 &&
+	      strcmp (contexts[i]->domain_dirname, GTK_LOCALEDIR) == 0)
+	    /* Input method may have a name in the GTK+ message catalog */
+	    translated_name = _(contexts[i]->context_name);
+	  else
+	    /* Input method has own message catalog */
+	    {
+	      bindtextdomain (contexts[i]->domain,
+			      contexts[i]->domain_dirname);
+#ifdef HAVE_BIND_TEXTDOMAIN_CODESET
+	      bind_textdomain_codeset (contexts[i]->domain, "UTF-8");
+#endif
+	      translated_name = dgettext (contexts[i]->domain, contexts[i]->context_name);
+	    }
+	}
+      else
+	/* Either domain or domain_dirname is NULL or "". We assume that
+	 * input method does not want a translated name in this case
+	 */
+	translated_name = contexts[i]->context_name;
+#else
+      translated_name = contexts[i]->context_name;
+#endif
       menuitem = gtk_radio_menu_item_new_with_label (group,
-                                                     contexts[i]->context_name);
+						     translated_name);
       
       if ((global_context_id == NULL && group == NULL) ||
           (global_context_id &&
@@ -489,15 +552,17 @@ gtk_im_multicontext_append_menuitems (GtkIMMulticontext *context,
         gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menuitem),
                                         TRUE);
       
-      group = gtk_radio_menu_item_group (GTK_RADIO_MENU_ITEM (menuitem));
+      group = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM (menuitem));
       
-      gtk_object_set_data (GTK_OBJECT (menuitem), "gtk-context-id",
-			   (char *)contexts[i]->context_id);
-      gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
-			  G_CALLBACK (activate_cb), context);
+      g_object_set_data (G_OBJECT (menuitem), "gtk-context-id",
+			 (char *)contexts[i]->context_id);
+      g_signal_connect (menuitem, "activate",
+			G_CALLBACK (activate_cb), context);
 
       gtk_widget_show (menuitem);
       gtk_menu_shell_append (menushell, menuitem);
     }
+
+  g_free (contexts);
 }
 
