@@ -28,14 +28,12 @@
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#include "config.h"
 #include "gdkconfig.h"
 
 #include "gtklayout.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
 #include "gtkmarshalers.h"
-#include "gtkalias.h"
 
 typedef struct _GtkLayoutChild   GtkLayoutChild;
 
@@ -59,6 +57,7 @@ enum {
   CHILD_PROP_Y
 };
 
+static void gtk_layout_class_init         (GtkLayoutClass *class);
 static void gtk_layout_get_property       (GObject        *object,
                                            guint           prop_id,
                                            GValue         *value,
@@ -70,6 +69,7 @@ static void gtk_layout_set_property       (GObject        *object,
 static GObject *gtk_layout_constructor    (GType                  type,
 					   guint                  n_properties,
 					   GObjectConstructParam *properties);
+static void gtk_layout_init               (GtkLayout      *layout);
 static void gtk_layout_finalize           (GObject        *object);
 static void gtk_layout_realize            (GtkWidget      *widget);
 static void gtk_layout_unrealize          (GtkWidget      *widget);
@@ -80,8 +80,6 @@ static void gtk_layout_size_allocate      (GtkWidget      *widget,
                                            GtkAllocation  *allocation);
 static gint gtk_layout_expose             (GtkWidget      *widget,
                                            GdkEventExpose *event);
-static void gtk_layout_add                (GtkContainer   *container,
-					   GtkWidget      *widget);
 static void gtk_layout_remove             (GtkContainer   *container,
                                            GtkWidget      *widget);
 static void gtk_layout_forall             (GtkContainer   *container,
@@ -112,7 +110,7 @@ static void gtk_layout_set_adjustment_upper (GtkAdjustment *adj,
 					     gdouble        upper,
 					     gboolean       always_emit_changed);
 
-G_DEFINE_TYPE (GtkLayout, gtk_layout, GTK_TYPE_CONTAINER)
+static GtkWidgetClass *parent_class = NULL;
 
 /* Public interface
  */
@@ -142,24 +140,6 @@ gtk_layout_new (GtkAdjustment *hadjustment,
   return GTK_WIDGET (layout);
 }
 
-/**
- * gtk_layout_get_bin_window:
- * @layout: a #GtkLayout
- * 
- * Retrieve the bin window of the layout used for drawing operations.
- * 
- * Return value: a #GdkWindow
- *
- * Since: 2.14
- **/
-GdkWindow*
-gtk_layout_get_bin_window (GtkLayout *layout)
-{
-  g_return_val_if_fail (GTK_IS_LAYOUT (layout), NULL);
-
-  return layout->bin_window;
-}
-
 /**
  * gtk_layout_get_hadjustment:
  * @layout: a #GtkLayout
@@ -244,10 +224,11 @@ gtk_layout_set_adjustments (GtkLayout     *layout,
   if (layout->hadjustment != hadj)
     {
       layout->hadjustment = hadj;
-      g_object_ref_sink (layout->hadjustment);
+      g_object_ref (layout->hadjustment);
+      gtk_object_sink (GTK_OBJECT (layout->hadjustment));
       gtk_layout_set_adjustment_upper (layout->hadjustment, layout->width, FALSE);
       
-      g_signal_connect (layout->hadjustment, "value-changed",
+      g_signal_connect (layout->hadjustment, "value_changed",
 			G_CALLBACK (gtk_layout_adjustment_changed),
 			layout);
       need_adjust = TRUE;
@@ -256,10 +237,11 @@ gtk_layout_set_adjustments (GtkLayout     *layout,
   if (layout->vadjustment != vadj)
     {
       layout->vadjustment = vadj;
-      g_object_ref_sink (layout->vadjustment);
+      g_object_ref (layout->vadjustment);
+      gtk_object_sink (GTK_OBJECT (layout->vadjustment));
       gtk_layout_set_adjustment_upper (layout->vadjustment, layout->height, FALSE);
       
-      g_signal_connect (layout->vadjustment, "value-changed",
+      g_signal_connect (layout->vadjustment, "value_changed",
 			G_CALLBACK (gtk_layout_adjustment_changed),
 			layout);
       need_adjust = TRUE;
@@ -279,7 +261,7 @@ gtk_layout_finalize (GObject *object)
   g_object_unref (layout->hadjustment);
   g_object_unref (layout->vadjustment);
 
-  G_OBJECT_CLASS (gtk_layout_parent_class)->finalize (object);
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 /**
@@ -388,7 +370,11 @@ gtk_layout_move_internal (GtkLayout       *layout,
                           gint             y)
 {
   GtkLayoutChild *child;
-
+  
+  g_return_if_fail (GTK_IS_LAYOUT (layout));
+  g_return_if_fail (GTK_IS_WIDGET (widget));
+  g_return_if_fail (widget->parent == GTK_WIDGET (layout));  
+  
   child = get_child (layout, widget);
 
   g_assert (child);
@@ -481,7 +467,7 @@ gtk_layout_set_size (GtkLayout     *layout,
   GtkWidget *widget;
   
   g_return_if_fail (GTK_IS_LAYOUT (layout));
-  
+
   widget = GTK_WIDGET (layout);
   
   g_object_freeze_notify (G_OBJECT (layout));
@@ -571,6 +557,33 @@ gtk_layout_thaw (GtkLayout *layout)
 
 /* Basic Object handling procedures
  */
+GType
+gtk_layout_get_type (void)
+{
+  static GType layout_type = 0;
+
+  if (!layout_type)
+    {
+      static const GTypeInfo layout_info =
+      {
+	sizeof (GtkLayoutClass),
+	NULL,		/* base_init */
+	NULL,		/* base_finalize */
+	(GClassInitFunc) gtk_layout_class_init,
+	NULL,		/* class_finalize */
+	NULL,		/* class_data */
+	sizeof (GtkLayout),
+	0,		/* n_preallocs */
+	(GInstanceInitFunc) gtk_layout_init,
+      };
+
+      layout_type = g_type_register_static (GTK_TYPE_CONTAINER, "GtkLayout",
+					    &layout_info, 0);
+    }
+
+  return layout_type;
+}
+
 static void
 gtk_layout_class_init (GtkLayoutClass *class)
 {
@@ -582,6 +595,8 @@ gtk_layout_class_init (GtkLayoutClass *class)
   widget_class = (GtkWidgetClass*) class;
   container_class = (GtkContainerClass*) class;
 
+  parent_class = g_type_class_peek_parent (class);
+
   gobject_class->set_property = gtk_layout_set_property;
   gobject_class->get_property = gtk_layout_get_property;
   gobject_class->finalize = gtk_layout_finalize;
@@ -593,57 +608,57 @@ gtk_layout_class_init (GtkLayoutClass *class)
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_X,
 					      g_param_spec_int ("x",
-                                                                P_("X position"),
-                                                                P_("X position of child widget"),
+                                                                _("X position"),
+                                                                _("X position of child widget"),
                                                                 G_MININT,
                                                                 G_MAXINT,
                                                                 0,
-                                                                GTK_PARAM_READWRITE));
+                                                                G_PARAM_READWRITE));
 
   gtk_container_class_install_child_property (container_class,
 					      CHILD_PROP_Y,
 					      g_param_spec_int ("y",
-                                                                P_("Y position"),
-                                                                P_("Y position of child widget"),
+                                                                _("Y position"),
+                                                                _("Y position of child widget"),
                                                                 G_MININT,
                                                                 G_MAXINT,
                                                                 0,
-                                                                GTK_PARAM_READWRITE));
+                                                                G_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
 				   PROP_HADJUSTMENT,
 				   g_param_spec_object ("hadjustment",
-							P_("Horizontal adjustment"),
-							P_("The GtkAdjustment for the horizontal position"),
+							_("Horizontal adjustment"),
+							_("The GtkAdjustment for the horizontal position"),
 							GTK_TYPE_ADJUSTMENT,
-							GTK_PARAM_READWRITE));
+							G_PARAM_READWRITE));
   
   g_object_class_install_property (gobject_class,
 				   PROP_VADJUSTMENT,
 				   g_param_spec_object ("vadjustment",
-							P_("Vertical adjustment"),
-							P_("The GtkAdjustment for the vertical position"),
+							_("Vertical adjustment"),
+							_("The GtkAdjustment for the vertical position"),
 							GTK_TYPE_ADJUSTMENT,
-							GTK_PARAM_READWRITE));
+							G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class,
 				   PROP_WIDTH,
 				   g_param_spec_uint ("width",
-						     P_("Width"),
-						     P_("The width of the layout"),
+						     _("Width"),
+						     _("The width of the layout"),
 						     0,
 						     G_MAXINT,
 						     100,
-						     GTK_PARAM_READWRITE));
+						     G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class,
 				   PROP_HEIGHT,
 				   g_param_spec_uint ("height",
-						     P_("Height"),
-						     P_("The height of the layout"),
+						     _("Height"),
+						     _("The height of the layout"),
 						     0,
 						     G_MAXINT,
 						     100,
-						     GTK_PARAM_READWRITE));
+						     G_PARAM_READWRITE));
   widget_class->realize = gtk_layout_realize;
   widget_class->unrealize = gtk_layout_unrealize;
   widget_class->map = gtk_layout_map;
@@ -652,23 +667,13 @@ gtk_layout_class_init (GtkLayoutClass *class)
   widget_class->expose_event = gtk_layout_expose;
   widget_class->style_set = gtk_layout_style_set;
 
-  container_class->add = gtk_layout_add;
   container_class->remove = gtk_layout_remove;
   container_class->forall = gtk_layout_forall;
 
   class->set_scroll_adjustments = gtk_layout_set_adjustments;
 
-  /**
-   * GtkLayout::set-scroll-adjustments
-   * @horizontal: the horizontal #GtkAdjustment
-   * @vertical: the vertical #GtkAdjustment
-   *
-   * Set the scroll adjustments for the layout. Usually scrolled containers
-   * like #GtkScrolledWindow will emit this signal to connect two instances
-   * of #GtkScrollbar to the scroll directions of the #GtkLayout.
-   */
   widget_class->set_scroll_adjustments_signal =
-    g_signal_new (I_("set-scroll-adjustments"),
+    g_signal_new ("set_scroll_adjustments",
 		  G_OBJECT_CLASS_TYPE (gobject_class),
 		  G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 		  G_STRUCT_OFFSET (GtkLayoutClass, set_scroll_adjustments),
@@ -820,9 +825,9 @@ gtk_layout_constructor (GType                  type,
   GObject *object;
   GtkAdjustment *hadj, *vadj;
   
-  object = G_OBJECT_CLASS (gtk_layout_parent_class)->constructor (type,
-								  n_properties,
-								  properties);
+  object = G_OBJECT_CLASS (parent_class)->constructor (type,
+						       n_properties,
+						       properties);
 
   layout = GTK_LAYOUT (object);
 
@@ -841,11 +846,14 @@ gtk_layout_constructor (GType                  type,
 static void 
 gtk_layout_realize (GtkWidget *widget)
 {
-  GtkLayout *layout = GTK_LAYOUT (widget);
   GList *tmp_list;
+  GtkLayout *layout;
   GdkWindowAttr attributes;
   gint attributes_mask;
 
+  g_return_if_fail (GTK_IS_LAYOUT (widget));
+
+  layout = GTK_LAYOUT (widget);
   GTK_WIDGET_SET_FLAGS (layout, GTK_REALIZED);
 
   attributes.window_type = GDK_WINDOW_CHILD;
@@ -862,7 +870,6 @@ gtk_layout_realize (GtkWidget *widget)
 
   widget->window = gdk_window_new (gtk_widget_get_parent_window (widget),
 				   &attributes, attributes_mask);
-  gdk_window_set_back_pixmap (widget->window, NULL, FALSE);
   gdk_window_set_user_data (widget->window, widget);
 
   attributes.x = - layout->hadjustment->value,
@@ -877,6 +884,7 @@ gtk_layout_realize (GtkWidget *widget)
   gdk_window_set_user_data (layout->bin_window, widget);
 
   widget->style = gtk_style_attach (widget->style, widget->window);
+  gtk_style_set_background (widget->style, widget->window, GTK_STATE_NORMAL);
   gtk_style_set_background (widget->style, layout->bin_window, GTK_STATE_NORMAL);
 
   tmp_list = layout->children;
@@ -890,10 +898,10 @@ gtk_layout_realize (GtkWidget *widget)
 }
 
 static void
-gtk_layout_style_set (GtkWidget *widget,
-                      GtkStyle  *old_style)
+gtk_layout_style_set (GtkWidget *widget, GtkStyle *old_style)
 {
-  GTK_WIDGET_CLASS (gtk_layout_parent_class)->style_set (widget, old_style);
+  if (GTK_WIDGET_CLASS (parent_class)->style_set)
+    (* GTK_WIDGET_CLASS (parent_class)->style_set) (widget, old_style);
 
   if (GTK_WIDGET_REALIZED (widget))
     {
@@ -901,11 +909,15 @@ gtk_layout_style_set (GtkWidget *widget,
     }
 }
 
-static void
+static void 
 gtk_layout_map (GtkWidget *widget)
 {
-  GtkLayout *layout = GTK_LAYOUT (widget);
   GList *tmp_list;
+  GtkLayout *layout;
+
+  g_return_if_fail (GTK_IS_LAYOUT (widget));
+
+  layout = GTK_LAYOUT (widget);
 
   GTK_WIDGET_SET_FLAGS (widget, GTK_MAPPED);
 
@@ -929,21 +941,30 @@ gtk_layout_map (GtkWidget *widget)
 static void 
 gtk_layout_unrealize (GtkWidget *widget)
 {
-  GtkLayout *layout = GTK_LAYOUT (widget);
+  GtkLayout *layout;
+
+  g_return_if_fail (GTK_IS_LAYOUT (widget));
+
+  layout = GTK_LAYOUT (widget);
 
   gdk_window_set_user_data (layout->bin_window, NULL);
   gdk_window_destroy (layout->bin_window);
   layout->bin_window = NULL;
 
-  GTK_WIDGET_CLASS (gtk_layout_parent_class)->unrealize (widget);
+  if (GTK_WIDGET_CLASS (parent_class)->unrealize)
+    (* GTK_WIDGET_CLASS (parent_class)->unrealize) (widget);
 }
 
 static void     
 gtk_layout_size_request (GtkWidget     *widget,
 			 GtkRequisition *requisition)
 {
-  GtkLayout *layout = GTK_LAYOUT (widget);
   GList *tmp_list;
+  GtkLayout *layout;
+
+  g_return_if_fail (GTK_IS_LAYOUT (widget));
+
+  layout = GTK_LAYOUT (widget);
 
   requisition->width = 0;
   requisition->height = 0;
@@ -965,10 +986,14 @@ static void
 gtk_layout_size_allocate (GtkWidget     *widget,
 			  GtkAllocation *allocation)
 {
-  GtkLayout *layout = GTK_LAYOUT (widget);
   GList *tmp_list;
+  GtkLayout *layout;
+
+  g_return_if_fail (GTK_IS_LAYOUT (widget));
 
   widget->allocation = *allocation;
+  
+  layout = GTK_LAYOUT (widget);
 
   tmp_list = layout->children;
 
@@ -1005,35 +1030,35 @@ gtk_layout_size_allocate (GtkWidget     *widget,
 }
 
 static gint 
-gtk_layout_expose (GtkWidget      *widget,
-                   GdkEventExpose *event)
+gtk_layout_expose (GtkWidget *widget, GdkEventExpose *event)
 {
-  GtkLayout *layout = GTK_LAYOUT (widget);
+  GtkLayout *layout;
+
+  g_return_val_if_fail (GTK_IS_LAYOUT (widget), FALSE);
+
+  layout = GTK_LAYOUT (widget);
 
   if (event->window != layout->bin_window)
     return FALSE;
-
-  GTK_WIDGET_CLASS (gtk_layout_parent_class)->expose_event (widget, event);
+  
+  (* GTK_WIDGET_CLASS (parent_class)->expose_event) (widget, event);
 
   return FALSE;
 }
 
-/* Container methods
+/* Container method
  */
-static void
-gtk_layout_add (GtkContainer *container,
-		GtkWidget    *widget)
-{
-  gtk_layout_put (GTK_LAYOUT (container), widget, 0, 0);
-}
-
 static void
 gtk_layout_remove (GtkContainer *container, 
 		   GtkWidget    *widget)
 {
-  GtkLayout *layout = GTK_LAYOUT (container);
   GList *tmp_list;
+  GtkLayout *layout;
   GtkLayoutChild *child = NULL;
+  
+  g_return_if_fail (GTK_IS_LAYOUT (container));
+  
+  layout = GTK_LAYOUT (container);
 
   tmp_list = layout->children;
   while (tmp_list)
@@ -1060,10 +1085,15 @@ gtk_layout_forall (GtkContainer *container,
 		   GtkCallback   callback,
 		   gpointer      callback_data)
 {
-  GtkLayout *layout = GTK_LAYOUT (container);
+  GtkLayout *layout;
   GtkLayoutChild *child;
   GList *tmp_list;
 
+  g_return_if_fail (GTK_IS_LAYOUT (container));
+  g_return_if_fail (callback != NULL);
+
+  layout = GTK_LAYOUT (container);
+
   tmp_list = layout->children;
   while (tmp_list)
     {
@@ -1099,6 +1129,10 @@ static void
 gtk_layout_adjustment_changed (GtkAdjustment *adjustment,
 			       GtkLayout     *layout)
 {
+  GtkWidget *widget;
+
+  widget = GTK_WIDGET (layout);
+
   if (layout->freeze_count)
     return;
 
@@ -1111,6 +1145,3 @@ gtk_layout_adjustment_changed (GtkAdjustment *adjustment,
       gdk_window_process_updates (layout->bin_window, TRUE);
     }
 }
-
-#define __GTK_LAYOUT_C__
-#include "gtkaliasdef.c"
