@@ -478,10 +478,18 @@ gtk_menu_window_size_request (GtkWidget      *window,
   if (private->have_position)
     {
       GdkScreen *screen = gtk_widget_get_screen (window);
-      gint screen_height = gdk_screen_get_height (screen);
+      GdkRectangle monitor;
+      gint monitor_num;
+      
+      monitor_num = gdk_screen_get_monitor_at_point (screen,
+						     private->x, private->y);
+      gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
 
-      if (private->y + requisition->height > screen_height)
-	requisition->height = screen_height - private->y;
+      if (private->y + requisition->height > monitor.y + monitor.height)
+	requisition->height = monitor.y + monitor.height - private->y;
+
+      if (private->y < monitor.y)
+	requisition->height -= monitor.y - private->y;
     }
 }
 
@@ -1998,7 +2006,7 @@ gtk_menu_key_press (GtkWidget	*widget,
 
   display = gtk_widget_get_display (widget);
     
-  g_object_get (G_OBJECT (gtk_widget_get_settings (widget)),
+  g_object_get (gtk_widget_get_settings (widget),
                 "gtk-menu-bar-accel", &accel,
 		"gtk-can-change-accels", &can_change_accels,
                 NULL);
@@ -2493,7 +2501,7 @@ draw_stay_up_triangle (GdkWindow *window,
                      clipbox.x, clipbox.y,
                      clipbox.width, clipbox.height);
   
-  g_object_unref (G_OBJECT (ugly_gc));
+  g_object_unref (ugly_gc);
 }
 #endif
 
@@ -2614,7 +2622,7 @@ gtk_menu_set_submenu_navigation_region (GtkMenu          *menu,
 	  menu->navigation_region = new_region;
 	}
 
-      g_object_get (G_OBJECT (gtk_widget_get_settings (GTK_WIDGET (menu))),
+      g_object_get (gtk_widget_get_settings (GTK_WIDGET (menu)),
 		    "gtk-menu-popdown-delay", &popdown_delay,
 		    NULL);
 
@@ -2712,7 +2720,7 @@ gtk_menu_position (GtkMenu *menu)
   
       if (y < monitor.y)
 	{
-	  scroll_offset -= y;
+	  scroll_offset += monitor.y - y;
 	  y = monitor.y;
 	}
     }
@@ -2720,13 +2728,21 @@ gtk_menu_position (GtkMenu *menu)
   /* FIXME: should this be done in the various position_funcs ? */
   x = CLAMP (x, monitor.x, MAX (monitor.x, monitor.x + monitor.width - requisition.width));
  
+  if (GTK_MENU_SHELL (menu)->active)
+    {
+      private = gtk_menu_get_private (menu);
+      private->have_position = TRUE;
+      private->x = x;
+      private->y = y;
+    }
+  
   if (y + requisition.height > monitor.y + monitor.height)
     requisition.height = (monitor.y + monitor.height) - y;
   
   if (y < monitor.y)
     {
-      scroll_offset -= y;
-      requisition.height -= -y;
+      scroll_offset += monitor.y - y;
+      requisition.height -= monitor.y - y;
       y = monitor.y;
     }
 
@@ -2736,16 +2752,7 @@ gtk_menu_position (GtkMenu *menu)
   gtk_window_move (GTK_WINDOW (GTK_MENU_SHELL (menu)->active ? menu->toplevel : menu->tearoff_window), 
 		   x, y);
 
-  if (GTK_MENU_SHELL (menu)->active)
-    {
-      private = gtk_menu_get_private (menu);
-      private->have_position = TRUE;
-      private->x = x;
-      private->y = y;
-
-      gtk_widget_queue_resize (menu->toplevel);
-    }
-  else
+  if (!GTK_MENU_SHELL (menu)->active)
     {
       gtk_window_resize (GTK_WINDOW (menu->tearoff_window),
 			 requisition.width, requisition.height);
@@ -2811,7 +2818,7 @@ gtk_menu_scroll_to (GtkMenu *menu,
   if (!menu->tearoff_active)
     {
       last_visible = menu->upper_arrow_visible;
-      menu->upper_arrow_visible = (view_height < menu_height && offset > 0);
+      menu->upper_arrow_visible = offset > 0;
       
       if (menu->upper_arrow_visible)
 	view_height -= MENU_SCROLL_ARROW_HEIGHT;
@@ -2827,7 +2834,7 @@ gtk_menu_scroll_to (GtkMenu *menu,
 	}
 
       last_visible = menu->lower_arrow_visible;
-      menu->lower_arrow_visible = (view_height < menu_height && offset < menu_height - view_height);
+      menu->lower_arrow_visible = offset < menu_height - view_height;
       
       if (menu->lower_arrow_visible)
 	view_height -= MENU_SCROLL_ARROW_HEIGHT;
@@ -2846,8 +2853,6 @@ gtk_menu_scroll_to (GtkMenu *menu,
 	y += MENU_SCROLL_ARROW_HEIGHT;
     }
 
-  offset = CLAMP (offset, 0, menu_height - view_height);
-
   /* Scroll the menu: */
   if (GTK_WIDGET_REALIZED (menu))
     gdk_window_move (menu->bin_window, 0, -offset);
@@ -3096,7 +3101,7 @@ gtk_menu_get_popup_delay (GtkMenuShell *menu_shell)
 {
   gint popup_delay;
 
-  g_object_get (G_OBJECT (gtk_widget_get_settings (GTK_WIDGET (menu_shell))),
+  g_object_get (gtk_widget_get_settings (GTK_WIDGET (menu_shell)),
 		"gtk-menu-popup-delay", &popup_delay,
 		NULL);
 
@@ -3165,6 +3170,24 @@ child_at (GtkMenu *menu,
   return child;
 }
 
+static gint
+get_menu_height (GtkMenu *menu)
+{
+  gint height;
+  GtkWidget *widget = GTK_WIDGET (menu);
+
+  height = widget->requisition.height;
+  height -= (GTK_CONTAINER (widget)->border_width + widget->style->ythickness) * 2;
+
+  if (menu->upper_arrow_visible && !menu->tearoff_active)
+    height -= MENU_SCROLL_ARROW_HEIGHT;
+
+  if (menu->lower_arrow_visible && !menu->tearoff_active)
+    height -= MENU_SCROLL_ARROW_HEIGHT;
+
+  return height;
+}
+
 static void
 gtk_menu_real_move_scroll (GtkMenu       *menu,
 			   GtkScrollType  type)
@@ -3178,6 +3201,7 @@ gtk_menu_real_move_scroll (GtkMenu       *menu,
       {
 	gint page_size = get_visible_size (menu);
 	gint old_offset;
+        gint new_offset;
 	gint child_offset = 0;
 	gboolean old_upper_arrow_visible;
 	gint step;
@@ -3199,7 +3223,11 @@ gtk_menu_real_move_scroll (GtkMenu       *menu,
 	menu_shell->ignore_enter = TRUE;
 	old_upper_arrow_visible = menu->upper_arrow_visible && !menu->tearoff_active;
 	old_offset = menu->scroll_offset;
-	gtk_menu_scroll_to (menu, menu->scroll_offset + step);
+
+        new_offset = menu->scroll_offset + step;
+        new_offset = CLAMP (new_offset, 0, get_menu_height (menu) - page_size);
+
+        gtk_menu_scroll_to (menu, new_offset);
 	
 	if (menu_shell->active_menu_item)
 	  {
