@@ -650,7 +650,8 @@ gtk_spin_button_size_request (GtkWidget      *widget,
 					   pango_context_get_language (context));
 
       digit_width = pango_font_metrics_get_approximate_digit_width (metrics);
-      digit_width = PANGO_PIXELS (digit_width);
+      digit_width = PANGO_SCALE *
+        ((digit_width + PANGO_SCALE - 1) / PANGO_SCALE);
 
       pango_font_metrics_unref (metrics);
       
@@ -662,15 +663,15 @@ gtk_spin_button_size_request (GtkWidget      *widget,
 
       string_len = compute_double_length (spin_button->adjustment->upper,
                                           spin_button->digits);
-      w = MIN (string_len, max_string_len) * digit_width;
+      w = PANGO_PIXELS (MIN (string_len, max_string_len) * digit_width);
       width = MAX (width, w);
       string_len = compute_double_length (spin_button->adjustment->lower,
 					  spin_button->digits);
-      w = MIN (string_len, max_string_len) * digit_width;
+      w = PANGO_PIXELS (MIN (string_len, max_string_len) * digit_width);
       width = MAX (width, w);
       
-      requisition->width = width;
-      if (interior_focus)
+      requisition->width = width + /* INNER_BORDER */ 2 * 2;
+      if (!interior_focus)
 	requisition->width += 2 * focus_width;
     }
 
@@ -955,16 +956,26 @@ static void
 gtk_spin_button_grab_notify (GtkWidget *widget,
 			     gboolean   was_grabbed)
 {
+  GtkSpinButton *spin = GTK_SPIN_BUTTON (widget);
+
   if (!was_grabbed)
-    gtk_spin_button_stop_spinning (GTK_SPIN_BUTTON (widget));
+    {
+      gtk_spin_button_stop_spinning (spin);
+      spin_button_redraw (spin);
+    }
 }
 
 static void
 gtk_spin_button_state_changed (GtkWidget    *widget,
 			       GtkStateType  previous_state)
 {
+  GtkSpinButton *spin = GTK_SPIN_BUTTON (widget);
+
   if (!GTK_WIDGET_IS_SENSITIVE (widget))
-    gtk_spin_button_stop_spinning (GTK_SPIN_BUTTON (widget));    
+    {
+      gtk_spin_button_stop_spinning (spin);    
+      spin_button_redraw (spin);
+    }
 }
 
 static gint
@@ -1006,12 +1017,15 @@ gtk_spin_button_stop_spinning (GtkSpinButton *spin)
   spin->timer = 0;
   spin->timer_step = spin->adjustment->step_increment;
   spin->timer_calls = 0;
+
+  spin->click_child = NO_ARROW;
+  spin->button = 0;
 }
 
 static void
 start_spinning (GtkSpinButton *spin,
 		GtkArrowType   click_child,
-		gfloat         step)
+		gdouble        step)
 {
   g_return_if_fail (click_child == GTK_ARROW_UP || click_child == GTK_ARROW_DOWN);
   
@@ -1083,7 +1097,7 @@ gtk_spin_button_button_release (GtkWidget      *widget,
 
   if (event->button == spin->button)
     {
-      guint click_child;
+      int click_child = spin->click_child;
 
       gtk_spin_button_stop_spinning (spin);
 
@@ -1093,7 +1107,7 @@ gtk_spin_button_button_release (GtkWidget      *widget,
 	      event->y <= widget->requisition.height &&
 	      event->x <= arrow_size + 2 * widget->style->xthickness)
 	    {
-	      if (spin->click_child == GTK_ARROW_UP &&
+	      if (click_child == GTK_ARROW_UP &&
 		  event->y <= widget->requisition.height / 2)
 		{
 		  gdouble diff;
@@ -1102,7 +1116,7 @@ gtk_spin_button_button_release (GtkWidget      *widget,
 		  if (diff > EPSILON)
 		    gtk_spin_button_real_spin (spin, diff);
 		}
-	      else if (spin->click_child == GTK_ARROW_DOWN &&
+	      else if (click_child == GTK_ARROW_DOWN &&
 		       event->y > widget->requisition.height / 2)
 		{
 		  gdouble diff;
@@ -1113,9 +1127,6 @@ gtk_spin_button_button_release (GtkWidget      *widget,
 		}
 	    }
 	}		  
-      click_child = spin->click_child;
-      spin->click_child = NO_ARROW;
-      spin->button = 0;
       spin_button_redraw (spin);
 
       return TRUE;
@@ -1485,7 +1496,7 @@ gtk_spin_button_default_input (GtkSpinButton *spin_button,
 {
   gchar *err = NULL;
 
-  *new_val = strtod (gtk_entry_get_text (GTK_ENTRY (spin_button)), &err);
+  *new_val = g_strtod (gtk_entry_get_text (GTK_ENTRY (spin_button)), &err);
   if (*err)
     return GTK_INPUT_ERROR;
   else
@@ -1681,7 +1692,7 @@ gtk_spin_button_get_adjustment (GtkSpinButton *spin_button)
 /**
  * gtk_spin_button_set_digits:
  * @spin_button: a #GtkSpinButton
- * @digits: the number of digits to be displayed for the spin button's value
+ * @digits: the number of digits after the decimal point to be displayed for the spin button's value
  * 
  * Set the precision to be displayed by @spin_button. Up to 20 digit precision
  * is allowed.
